#!/usr/bin/perl -w

# Copyright 2010, 2011 Kevin Ryde

# This file is part of Perl-Critic-Pulp.
#
# Perl-Critic-Pulp is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3, or (at your option) any
# later version.
#
# Perl-Critic-Pulp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with Perl-Critic-Pulp.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use lib '.';
use lib 'devel';
use version_check 1.1;

{
  my $v = version_check->VERSION;
  print "ref: ",ref $v,"\n";
}
{
  my $ret = version_check->VERSION('0');
  print "ret $ret\n";
}
{
  my $ret = version_check->VERSION('0');
  print "ret $ret\n";
}
print "done\n";
